//*****************************************************************************
//*****************************************************************************
//  FILENAME:   DigitalIO.c
//  Version: 1.0, 2006/10/25 
//  Generated by Andy Kiser
//
//  DESCRIPTION:  Processing digital inputs/outputs controlled by USB
//
//-----------------------------------------------------------------------------
// Originally authored by Toradex AG. This work is now available under the terms and conditions
// of the Creative Commons License 'Attribution CC BY', details of which can be found here:
// http://creativecommons.org/licenses/by/3.0/
//*****************************************************************************
//*****************************************************************************
#include <m8c.h>        // part specific constants and macros
#include "feature.h"
#include "parameters.h"
#include "types_n_macros.h"

// define registers according to the port definitions above
#define PRT_IN_DM2	PRT5DM2
#define PRT_IN_DM1	PRT5DM1
#define PRT_IN_DM0	PRT5DM0
#define PRT_IN_DR	PRT5DR

#define PRT_OUT_DM2	PRT4DM2
#define PRT_OUT_DM1	PRT4DM1
#define PRT_OUT_DM0	PRT4DM0
#define PRT_OUT_DR	PRT4DR

#define PRT_LED_DM2	PRT0DM2
#define PRT_LED_DM1	PRT0DM1
#define PRT_LED_DM0	PRT0DM0
#define PRT_LED_DR	PRT0DR

#define MASK_LED	0x02	// %0000'0010  -> LED connected to PORT0[1]

// state variables
static BYTE DigOut_Shadow;

//-----------------------------------------------------------------------------
// Initialize Digital Input / Output pins
//-----------------------------------------------------------------------------
void DigitalIO_Start()
{
	// -- initialize "ShowUp" LED
	GPIO_DM_OPENDRAIN(LED_SHOWUP)
	GPIO_HIGH(LED_SHOWUP)
	
	// -- initialize Digital Outputs
	// Set Port4 Drivemode to [001b] (Strong Digital Output)
	PRT_OUT_DM2 = 0x00;	// set driver mode and strength
	PRT_OUT_DM1 = 0x00;
	PRT_OUT_DM0 = 0xFF;
	
	DigOut_Shadow = 0x00;
	PRT_OUT_DR  = DigOut_Shadow;	// set all 8 outputs to 0

	// -- initialize Digital Inputs
	// Set Port5 Drivemode to [010b] (Digital Input)
	PRT_IN_DM2 = 0x00;	// set driver mode and strength
	PRT_IN_DM1 = 0xFF;
	PRT_IN_DM0 = 0x00;
}

//-----------------------------------------------------------------------------
// Prepare for Suspend
//-----------------------------------------------------------------------------
void DigitalIO_Suspend()
{
	// -- Set all Drivemodes to [110b] (HiZ, digital in disabled for zero power)
	// particular use (e.g. "smile" LED)
	GPIO_DM_HIGHZ(LED_SHOWUP)
	
	// initialize Digital Outputs
	PRT_OUT_DM2 = 0xFF;	// set driver mode and strength
	PRT_OUT_DM1 = 0xFF;
	PRT_OUT_DM0 = 0x00;
	
	// initialize Digital Inputs
	PRT_IN_DM2 = 0xFF;	// set driver mode and strength
	PRT_IN_DM1 = 0xFF;
	PRT_IN_DM0 = 0x00;
}

//-----------------------------------------------------------------------------
// Resume from Suspend 
//-----------------------------------------------------------------------------
void DigitalIO_Resume()
{
	// -- restore drive modes
	
	// -- initialize "ShowUp" LED
	GPIO_DM_OPENDRAIN(LED_SHOWUP)
	GPIO_HIGH(LED_SHOWUP)
	
	// -- initialize Digital Outputs
	// Set Port4 Drivemode to [001b] (Strong Digital Output)
	PRT_OUT_DM2 = 0x00;	// set driver mode and strength
	PRT_OUT_DM1 = 0x00;
	PRT_OUT_DM0 = 0xFF;

	// -- initialize Digital Inputs
	// Set Port2 Drivemode to [010b] (Digital Input)
	PRT_IN_DM2 = 0x00;	// set driver mode and strength
	PRT_IN_DM1 = 0xFF;
	PRT_IN_DM0 = 0x00;
}

//-----------------------------------------------------------------------------
// ShowUp state machine
// call this function once every millisecond
//-----------------------------------------------------------------------------
void FSM_LEDmode ()
{
	static WORD Count_ms = 0;
	BYTE		LedStatus;

	#define PAR8_LEDMODE 1
	Get_Param_8(PAR8_LEDMODE, VOLATILE, &LedStatus);
	
	switch (LedStatus)
	{
	// ---- LED OFF
	case 0:
		GPIO_HIGH(LED_SHOWUP) // LED off
		break;
		
	// ---- LED ON
	case 1:
		GPIO_LOW(LED_SHOWUP) // LED on
		break;
		
	// ---- LED blink slow
	case 2:
		if (Count_ms == 0)
		{
			GPIO_LOW(LED_SHOWUP) // LED on
			Count_ms = 500;
		}
		else if (Count_ms == 450)
			GPIO_HIGH(LED_SHOWUP) // LED off
		break;
	// ---- LED blink fast

	case 3:
		if (Count_ms == 0)
		{
			GPIO_LOW(LED_SHOWUP) // LED on
			Count_ms = 150;
		}
		else if (Count_ms == 100)
			GPIO_HIGH(LED_SHOWUP) // LED off
		else;
		break;
	
	// ---- LED blink quad
	case 4:
		switch (Count_ms)
		{
		case 2000:
		case 1800:
		case 1600:
		case 1400:
				GPIO_LOW(LED_SHOWUP) // LED on
				break;

		case 1950:
		case 1750:
		case 1550:
		case 1350:
				GPIO_HIGH(LED_SHOWUP) // LED off
				break;
				
		case 0:
				Count_ms = 2001;
		
		}
	}
	if (Count_ms != 0)
		Count_ms--;
}

//-----------------------------------------------------------------------------
// Function to set the value of the 8 digital outputs
//-----------------------------------------------------------------------------
void DigitalOut_Set (BYTE msk_and, BYTE msk_or, BYTE msk_xor)
{
	DigOut_Shadow = ( (DigOut_Shadow & msk_and) | msk_or ) ^ msk_xor;
	PRT_OUT_DR = DigOut_Shadow;
}


//-----------------------------------------------------------------------------
// Function returns value of the 8 digital inputs
//-----------------------------------------------------------------------------
BYTE DigitalIn_Get (void)
{
	return PRT_IN_DR;
}
