//----------------------------------------------------------------------------
// C main line for "Oak" Sensor
// Version: 1.0, 2007/03/08 
//
///----------------------------------------------------------------------------
// Originally authored by Toradex AG. This work is now available under the terms and conditions
// of the Creative Commons License 'Attribution CC BY', details of which can be found here:
// http://creativecommons.org/licenses/by/3.0/
///----------------------------------------------------------------------------
#define __DEBUG__

#include <m8c.h>        // part specific constants and macros
#include <string.h>
#include "PSoCAPI.h"    // PSoC API definitions for all User Modules
#include "PSoCgpioInt.h"
#include "USBsleeptimer.h"
#include "feature.h"
#include "parameters.h"
#include "DigitalIO.h"
#include "sensor.h"
#include "types_n_macros.h"
//-----------------------------------------------------------------------------
// global variables
//-----------------------------------------------------------------------------
extern BYTE SOF_Flag;	// Set in SOF_ISR
extern BYTE	USB_INTERFACE_0_IN_RPT_DATA[IN_REPORT_SIZE];


//-----------------------------------------------------------------------------
// function to send input reports to the host
// call this function once every millisecond to maintain proper timings
//-----------------------------------------------------------------------------
void USB_IN_Report()
{
	static tInReport	Last_InReport;
	static WORD			ReportRate_Countdown;
	BOOL				Send;
	WORD				FrameNr;
	eReportMode			ReportMode;
		
	// -- update counter for fixed report rate
	if (ReportRate_Countdown != 0)
		ReportRate_Countdown--;
		
	// -- decide if IN report has to be sent, depending on the report mode
	Get_Param_8(PAR8_REPORTMODE, VOLATILE, (BYTE*)&ReportMode);
	switch (ReportMode)
	{
		case AfterSampling:
			Send = NewSample;
			break;
			
		case AfterChange:
			if (NewSample)
				Send = memcmp( (BYTE*)(&InReport)+2, (BYTE*)(&Last_InReport)+2, IN_REPORT_SIZE-2 ) != 0;
			else
				Send = FALSE;
			break;
			
		case FixedRate:
			Send = (ReportRate_Countdown == 0);
			break;
	}
	
	// -- send report data
	if (Send)
	{
		// readout absolute frame number --> transmit as timestamp
		do
			FrameNr = USB_SOF0 + 256*USB_SOF1;
		while ((FrameNr / 256) != USB_SOF1);	// protect against overflow during readout
		
		Get_Param_16(PAR16_REPORTRATE_MS, VOLATILE, &ReportRate_Countdown);

		// numeric values in reports are represented in little-endian format
		// with the least significant byte at the lowest address.			
		InReport.FrameNr_LSB = FrameNr & 0xFF;
		InReport.FrameNr_MSB = FrameNr / 0x100;

		USB_LoadInEP(EP_IN, (BYTE*)&InReport, IN_REPORT_SIZE, USB_TOGGLE);

		// update last transmitted sensor values (for detecting changes)
		memcpy((BYTE*)(&Last_InReport)+2, (BYTE*)(&InReport)+2, IN_REPORT_SIZE-2);
		// copy data to IN report. Necessary to support GET_REPORT request
		memcpy(USB_INTERFACE_0_IN_RPT_DATA, &InReport, IN_REPORT_SIZE);

		NewSample=FALSE;
	}
}

//-----------------------------------------------------------------------------
// function to read OUT reports from the host
// call this function once every millisecond to maintain proper timings
//-----------------------------------------------------------------------------
void USB_OUT_Report(BYTE *Buffer, BYTE BufSize, BYTE *siz)
{
	if ( USB_bGetEPState(EP_OUT) == OUT_BUFFER_FULL)
	{
		*siz = USB_bReadOutEP(EP_OUT, Buffer, BufSize);
		USB_EnableOutEP(EP_OUT);
	}
}

//-----------------------------------------------------------------------------
// 1) Suspend the CPU and peripherals, including sensor
// 2) wait for an interrupt to wake the CPU
// 3) Resume to normal operation
//-----------------------------------------------------------------------------
void SuspendResume()
{
	M8C_DisableGInt;

	// -- prepare sleep
		// reset sensor to stop current measurement
	Sensor_Suspend();
		// configure I/Os for minimum current consumption
	DigitalIO_Suspend();

		// activate "USB wakeup" interrupt
	M8C_EnableIntMask( INT_MSK2, INT_MSK2_WAKEUP );
		// set sleep interval to 1sec, Osc-Freq to 3MHz, Bandgap OFF
	
	OSC_CR0 = OSC_CR0_CPU_3MHz | OSC_CR0_SLEEP_1Hz; 
//	OSC_CR0 = OSC_CR0_CPU_3MHz | OSC_CR0_SLEEP_64Hz; 
		// disable SleeptTimer interrupt
	M8C_DisableIntMask( INT_MSK0, INT_MSK0_SLEEP );

	// -- enter sleep
	M8C_Sleep
	M8C_EnableGInt;
	
	// -- zzzzz... zzz... zzzzzz...
	// -- ...
	// -- Rrrriiiiing...   Rrrrriiiiiing... Rrrriiiing...
	
	// enable SleeptTimer interrupt
	M8C_EnableIntMask( INT_MSK0, INT_MSK0_SLEEP );
		// if PLL mode was enabled (sync to external XTAL oscillator) it is recommended 
		// to wait 10ms after resume before switching to 12MHz
		// -> no need to do this, as we do not use PLL mode <- 

		// set sleep interval to 1.95msec, Osc-Freq to 12MHz, Bandgap OFF
	OSC_CR0 = OSC_CR0_CPU_12MHz | OSC_CR0_SLEEP_512Hz; 
	
		// restart sensor measurement
	Sensor_Resume();
		// restore I/O settings for proper operation
	DigitalIO_Resume();
}

//-----------------------------------------------------------------------------
// void main(void)
//
//	main application for the USB attached SHT1x sensor
//-----------------------------------------------------------------------------
void main() 
{
	DigitalIO_Suspend();
	USB_Start(0, USB_5V_OPERATION);


	M8C_EnableGInt;

	Sensor_Start();
	SleepTimer_Start();
	DigitalIO_Start();
	Feature_Start();
	NofOutBytes = 0;

	#ifdef __DEBUG__
	Sensor_Debug();
	#endif

	// wait for USB enumeration, enter low-power  mode if sensor is hot-plugged into suspended host
	while ( !USB_bGetConfiguration() )
		if (GotoSleep)
			SuspendResume();
	USB_INT_REG |= USB_INT_SOF_MASK;
	USB_EnableOutEP(EP_OUT);

	while(1)
	{
		if (SOF_Flag) 
		{	// get here once every millisecond, as long as USB is active
			SOF_Flag = 0;

			Sensor_FSM_Measure();
	
			USB_OUT_Report((BYTE*)&OutReport, OUT_REPORT_SIZE, &NofOutBytes);
			USB_IN_Report();
			FSM_LEDmode();
	
		} 

		ProcessFeatureReports();
			
		// the following code is executed independent of the millisecond tick rate.
		if (GotoSleep)
			SuspendResume();
	}
}

