//*****************************************************************************
//*****************************************************************************
//  FILENAME:   sensor.c
//  Version: 1.0, 2007/03/08
//  Generated by Andy Kiser
//
//  DESCRIPTION:  sensor specific functions 
//
//-----------------------------------------------------------------------------
// Originally authored by Toradex AG. This work is now available under the terms and conditions
// of the Creative Commons License 'Attribution CC BY', details of which can be found here:
// http://creativecommons.org/licenses/by/3.0/
//*****************************************************************************
//*****************************************************************************
//#define __DEBUG__

#include <m8c.h>        // part specific constants and macros
#include "string.h"
#include "sensor.h"
#include "psocgpioint.h"
#include "psocapi.h"
#include "feature.h"
#include "parameters.h"
#include "types_n_macros.h"


#ifdef __DEBUG__
#pragma ioport  UART_TX_CONTROL_REG:    0x02b               // (DxBxxCR0) Control register
BYTE            UART_TX_CONTROL_REG;
#pragma ioport  UART_TX_OUTPUT_REG: 0x12a                   // (DxBxxOU) Output register
BYTE            UART_TX_OUTPUT_REG;
#endif

//----------------------------------------
// Sensor parameters
// Factory Defaults are defined here (NUM_xxx are defined in "sensor.h")
//----------------------------------------
const BYTE FactoryDefault_Param_8   [NUM_PARAM_8] =
{
	0x02,		// report mode
	0x00,		// LED mode
};

const WORD FactoryDefault_Param_16  [NUM_PARAM_16] = 
{
	500,		// report rate [ms]
	500,		// sample rate [ms]
	50000		// sample rate [us]	(not used for this sensor)
};

const DWORD FactoryDefault_Param_32  [NUM_PARAM_32] =
{
	0x00000000	// currently no 32bit parameters
};

const tParam_168 FactoryDefault_Param_168 [NUM_PARAM_168] =
{
	"Quad Relay",	// user device name
	"Frame Number", // user name Channel 0
	"Relay 0",		
	"Relay 1",
	"Relay 2",
	"Relay 3"
};


//----------------------------------------
// this module's global variables
//----------------------------------------
tInReport	InReport;
tOutReport	OutReport;
tInReport	InReport;
BYTE	NewSample;   // code in this file must set NewSample to TRUE, when new data is ready to be transferred to the host
BYTE	NofOutBytes; // number of bytes that were read from the last OUT report

extern BYTE SOF_Flag;
//----------------------------------------
// constant definitions
//----------------------------------------

// Magellan Errors
#define MGL_NOERROR	0
#define MGL_PILOTRESET 1
#define MGL_INVALIDINSTRUCTION 2
#define MGL_INVALIDAXIS 3
#define MGL_INVALIDPARAMETER 4
#define MGL_TRACERUNNING 5
#define MGL_RESERVED 6
#define MGL_BLOCKBOUNDEXCEEDED 7
#define MGL_TRACEZERO 8
#define MGL_BADCHECKSUM 9
#define MGL_NOTPRIMARYPORT 10
#define MGL_NEGATIVEVELOCITY 11
#define MGL_SCURVECHANGE 12
#define MGL_LIMITEVENTPENDING 13
#define MGL_MOVEINTOLIMIT 14
#define MGL_BUSY 16 /* This is my own extension */
#define MGL_COMMERROR 17 /* This is my own extension */

// custom Magellan Commands (only for communication USB host <-> USB device
#define MGLC_SET_CURRENTLIMIT 0x16
#define MGLC_SET_CURRENTLIMIT_NONVOLATILE 0x20
#define MGLC_GET_CURRENTLIMIT 0x17
#define MGLC_SET_INITIALIZE_STEPPER 0x18
#define MGLC_SET_INITIALIZE_STEPPER_NONVOLATILE 0x22
#define MGLC_GET_INITIALIZE_STEPPER 0x19

// Magellan Motor Types
#define MOT_BRUSHLESS_DC_3PH	0
#define MOT_BRUSHLESS_DC_2PH	1
#define MOT_STEPPER_3PH			2
#define MOT_STEPPER_2PH			3
#define MOT_PULSE_DIR			4
#define MOT_RSVD5				5
#define MOT_RSVD6				6
#define MOT_DC_BRUSH			7


#define RELAY_MASK (RELAY0_MASK | RELAY1_MASK | RELAY2_MASK | RELAY3_MASK)

//----------------------------------------
// type definitions
//----------------------------------------

typedef enum 
{	
	eOUT,
	eIN
} tFSM_Measure;



//----------------------------------------
// this module's global variables
//----------------------------------------

tFSM_Measure	FSM, StateAfterError;
WORD			Countdown_1ms;

BYTE			Relay_StandbyAction[4];

// possible actions for each relay
#define RLY_OFF			0
#define RLY_ON			1
#define RLY_TOGGLE		2
#define RLY_NOCHANGE	3

// store original drivemodes during suspend
BYTE			DM0_2, DM0_1, DM0_0;
BYTE			DM2_2, DM2_1, DM2_0;
BYTE			PORT_RELAY;

//----------------------------------------
// function prototypes
//----------------------------------------
void Set_RelayPins(BYTE Action[4]);	

//-----------------------------------------------------------------------------
// Initializes the Sensor 
//-----------------------------------------------------------------------------
void Sensor_Start (void)
{
	BYTE i;
	
	RELAY0_Data_ADDR &= ~RELAY_MASK;
	FSM = eOUT;
	for(i=0; i<4; i++)
		Relay_StandbyAction[i] = RLY_OFF;

}

//-----------------------------------------------------------------------------
// Prepare Sensor for Suspend
//-----------------------------------------------------------------------------
void Sensor_Suspend()
{
	BYTE ParNr,
		 ILIM;
	
	// store original drivemodes during  suspend
	DM0_2 = PRT0DM2;
	DM0_1 = PRT0DM1;
	DM0_0 = PRT0DM0;
	DM2_2 = PRT2DM2;
	DM2_1 = PRT2DM1;
	DM2_0 = PRT2DM0;
	PORT_RELAY = RELAY0_Data_ADDR;
		
	Set_RelayPins(Relay_StandbyAction);	

	// set all I/Os to/from the sensor to High-Z Analog (=110), 
	// except the relay outputs
	PRT0DM2  = 0xFF;
	PRT0DM1  = 0xFF;
	PRT0DM0  = 0x00;
	PRT2DM2  |= ~RELAY_MASK; // leave RELAY pins unmodified 
	PRT2DM1  |= ~RELAY_MASK;
	PRT2DM0  &=  RELAY_MASK;
}

//-----------------------------------------------------------------------------
// Resume Sensor from Suspend mode
//-----------------------------------------------------------------------------
void Sensor_Resume(void)
{
	// restore drivemodes for GPIOs to/from the sensor
	PRT2DM2 = DM2_2;
	PRT2DM1 = DM2_1;
	PRT2DM0 = DM2_0;

	PRT0DM2 = DM0_2;
	PRT0DM1 = DM0_1;
	PRT0DM0 = DM0_0;

	// set Relay pins to operational state
	RELAY0_Data_ADDR = PORT_RELAY;
}
//-----------------------------------------------------------------------------
// Built-in self test
//-----------------------------------------------------------------------------
void BIST(void)
{
	BYTE i;
	WORD d;
	BYTE Actions[4][4] = {
							{2, 3, 3, 3},
							{3, 2, 3, 3},
							{3, 3, 2, 3},
							{3, 3, 3, 2}
						  };
	i=0;
	for(;;)
	{
		Set_RelayPins( Actions[i&3] );
		i++;

		for (d=0; d<500; d++);	
	}
	
	// no useful function implemented yet
}

//-----------------------------------------------------------------------------
// Function that will be called when a FEATURE REPORT is received that 
// cannot be handled with the standard functionality in feature.c
// The feature report data is available in the "FEATURE_RPT" variable
//
// --> see feature.c for an example how to parse FEATURE_RPT
//
//	return Value: number of Parameter bytes (1-31) that are 
//				  returned in the IN report
//-----------------------------------------------------------------------------
BYTE Sensor_Specific_Feature(void)
{
	// Start Builtin Self Test
	if  ( (!FEATURE_RPT.OUT.Get_nSet) && (FEATURE_RPT.OUT.Target == eAction) &&
	      (FEATURE_RPT.OUT.ParamSize == 0) && (FEATURE_RPT.OUT.Index_LSB == 0x80) )
	{	      
		BIST();
		return 1; 
	}		
	if  ( (FEATURE_RPT.OUT.Target == eController)&&
	      (FEATURE_RPT.OUT.ParamSize == 4) 		 &&
	      (FEATURE_RPT.OUT.Index_LSB == 0x00)    &&
		  (FEATURE_RPT.OUT.Index_MSB == 0x00)     	)
	{
		// Features Out Report
		if (!FEATURE_RPT.OUT.Get_nSet)
		{
			// Change state of relays according to the out report data
			Set_RelayPins((BYTE*)&FEATURE_RPT.OUT.Param[0]);

			// store information for entering standby
			Relay_StandbyAction[0] = FEATURE_RPT.OUT.Param[0] >> 4;
			Relay_StandbyAction[1] = FEATURE_RPT.OUT.Param[1] >> 4;
			Relay_StandbyAction[2] = FEATURE_RPT.OUT.Param[2] >> 4;
			Relay_StandbyAction[3] = FEATURE_RPT.OUT.Param[3] >> 4;
		}			
		// Set and Get give the same Feature In Report 	


		// Report Relay Status back to the Host
		FEATURE_RPT.IN.Param[0] = ( (RELAY0_Data_ADDR & RELAY0_MASK)? 1 : 0 );
		FEATURE_RPT.IN.Param[1] = ( (RELAY0_Data_ADDR & RELAY1_MASK)? 1 : 0 );
		FEATURE_RPT.IN.Param[2] = ( (RELAY0_Data_ADDR & RELAY2_MASK)? 1 : 0 );
		FEATURE_RPT.IN.Param[3] = ( (RELAY0_Data_ADDR & RELAY3_MASK)? 1 : 0 );
		return 4;
	}
	return 1; 
}


//-----------------------------------------------------------------------------
// Set the Relay output pins
//-----------------------------------------------------------------------------
void Set_RelayPins(BYTE Action[4])
{
	BYTE Relay_Shadow;
	BYTE Mask;
	BYTE i;

	// set the current limit pins to the desired value	
	Relay_Shadow = RELAY0_Data_ADDR;

	for (i=0; i<4; i++)
	{
		switch (i)
		{
			case 0:
				Mask = RELAY0_MASK;
				break;
			case 1:
				Mask = RELAY1_MASK;
				break;
			case 2:
				Mask = RELAY2_MASK;
				break;
			case 3:
				Mask = RELAY3_MASK;
				break;
		}	

		switch (Action[i]&0x0f)
		{
			case RLY_OFF:
				Relay_Shadow &= ~Mask;
				break;
			case RLY_ON:
				Relay_Shadow |= Mask;
				break;
			case RLY_TOGGLE:
				Relay_Shadow ^= Mask;
			case RLY_NOCHANGE:
			default:
				// do nothing
				break;		
		}
	}
	
	RELAY0_Data_ADDR = Relay_Shadow;	
}

//-----------------------------------------------------------------------------
// Function that will be called when a FEATURE REPORT is received that 
// sets a 8/16/32/168bit parameter in feature.c
// The feature report data is available in the "FEATURE_RPT" variable
//
// --> see feature.c for an example how to parse FEATURE_RPT
//-----------------------------------------------------------------------------
void Sensor_Specific_Parameter(BYTE *NumberOfReturnedBytes)
{
	NumberOfReturnedBytes = 0;
}


//-----------------------------------------------------------------------------
// void Sensor_FSM_Measure(void)
//
//	finite state machine to control the measurement process for 
//  the sensor
//
//  IMPORTANT: 	call this routine once every millisecond. Otherwise the timing
//				parameters cannot be met.
//-----------------------------------------------------------------------------
void Sensor_FSM_Measure(void) 
{
	BYTE i;
	BYTE Mask;
		
	if (Countdown_1ms != 0)							// decrement millisecond counter
		Countdown_1ms--;
													// check if new instruction transmitted during currently active instruction
	// process actual state machine
	switch (FSM)
	{
		//----------------
		case eOUT: 
		//----------------
			if (NofOutBytes >0) 
			{
				Set_RelayPins((BYTE*)&OutReport);			// Change state of relays
				
				for (i=0; i<4; i++)					// store information for entering standby 
					Relay_StandbyAction[i] = OutReport.Relay[i] >> 4;
				NofOutBytes = 0;  					// Mark OUT report bytes as processed
				
				for (i=0; i<4; i++)						// Report Relay Status back to the Host
				{
					switch (i)
					{
						case 0:
							Mask = RELAY0_MASK;
							break;
						case 1:
							Mask = RELAY1_MASK;
							break;
						case 2:
							Mask = RELAY2_MASK;
							break;
						case 3:
							Mask = RELAY3_MASK;
							break;
					}	
					
					InReport.Relay[i] = ( (RELAY0_Data_ADDR & Mask)? 1 : 0 );
				}
				NewSample = TRUE;
			}					
			break;
	}
}


//-----------------------------------------------------------------------------
void Sensor_Debug(void)
{
/* currently no debug function */
}

// end of sensor.c